#ifndef __GUI_H__
#define __GUI_H__

#include <gtk/gtk.h>

#define SBAW_NRFILES 8 // Max number of source files

//------------------------------------------------------------
//
// Create structures to generically access the pic-processor
//

class pic_processor;

//
// Here's a list of all the types of windows that are supported
//

enum window_types {
  WT_register_window,
  WT_status_bar,
  WT_sfr_window,
  WT_watch_window,
  WT_stack_window,
  WT_asm_source_window,
  WT_opcode_source_window,
  WT_list_source_window
};

//
// Here's a list of all the categories of windows that are supported
//
enum window_category {
  WC_misc,
  WC_source,
  WC_data
};

//
// This structure will cross reference the data in the simulator
// to its gui representation. There are cases when the same data
// appears in more than one place (e.g. the status register is in
// both the status bar and register windows). gpsim accomodates this
// with a singly-linked list. In other words, for each data element
// that is presented graphically there's a pointer within the simulator
// to reference it. The simulator keeps a linked listed of pointers
// to all instances of these graphical representations

struct cross_reference_to_gui {
  window_types parent_window_type;
  gpointer     parent_window;
  gpointer     data;
  void         (*update)  (cross_reference_to_gui  *_this,int new_value);
};


//
// Forward reference
//

struct _gui_processor;


//
// Make the declarations pretty
//

typedef struct _gui_processor GUI_Processor;

//
// GUI_Object 
//  All window attributes that are common are placed into the GUI_Object
// structure. This structure is then include in each of the other structures.
// It's also the very first item in these 'derived' structures. Consequently a
// pointer to one object may be type cast into another. 
//

struct _gui_object {
  GUI_Processor *gp;
  GtkWidget *window;
  window_category wc;

  char *name;

  // Window geometry. This info is saved when the window associated
  // with this gui object is hidden. Note: gtk saves the window origin
  // (x,y) but doesn't save the size (width,height).
  int x,y,width,height;

  // A pointer to a function that will allow the window associated
  // with this gui object to be viewable or hidden.

  int (* change_view) (struct _gui_object *_this, int view_state);
};

typedef struct _gui_object GUI_Object;

//
// A 'register' has two attributes as far as the gui is concerned:
//   1) its location and 2) value that is being displayed
//

struct _register {
  int row;        // row & col in register window
  int col;
  int value;      // value displayed in register window
};

typedef struct _register Register;

//
// A 'labeled entry' is an object consisting of gtk entry
// widget that is labeled (with a gtk lable widget) and 
// has information about its parent.
//

struct _labeled_entry {
  GtkWidget *label;
  GtkWidget *entry;

  int value;         // value displayed

  gpointer parent;   // a pointer to the owner
  int handle_id;     // unique identifier
};

typedef struct _labeled_entry labeled_entry;


//
// The register window 
//

struct _Register_Window {
    GUI_Object     gui_obj;

    //  GUI_Processor *gp;
    Register **registers;
    GtkSheet *register_sheet;
};

typedef struct _Register_Window Register_Window;

//
// The Status Bar window 
//

struct _StatusBar_Window {
  GUI_Processor *gp;

  labeled_entry *status;
  labeled_entry *W;
  labeled_entry *pc;
  labeled_entry *cycles;


};

typedef struct _StatusBar_Window StatusBar_Window;

struct _SourceBrowser_Window {
    GUI_Object gui_obj;
    int wt;
    GtkWidget *vbox; /* for children to put widgets in */
};

typedef struct _SourceBrowser_Window SourceBrowser_Window;


struct breakpoint_info {
    int address;
    GtkWidget *widget;
};

//
// The Source Assembler Browser Data
//
struct _SourceBrowserAsm_Window {
    SourceBrowser_Window sbw;

    struct breakpoint_info breakpoint[MAX_BREAKPOINTS];
    int fixed_offset;

    // We need one of theese for each source file
    GtkAdjustment *source_layout_adj[SBAW_NRFILES];
    GtkWidget *source_layout[SBAW_NRFILES];
    GtkWidget *source_text[SBAW_NRFILES];
    int pageindex_to_fileid[SBAW_NRFILES];
    GtkWidget *source_pcwidget[SBAW_NRFILES];
    GtkWidget *notebook_child[SBAW_NRFILES];
    
    GdkBitmap *mask;
//    GtkWidget *source_scrollwin;
//    GtkWidget *source_frame;
    GtkWidget *notebook;

    GtkStyle symbol_text_style;       // for symbols in .asm display
    GtkStyle label_text_style;        // for label in .asm display
    GtkStyle instruction_text_style;  // for instruction in .asm display
    GtkStyle number_text_style;       // for numbers in .asm display
    GtkStyle comment_text_style;      // for comments in .asm display
    GtkStyle default_text_style;      // the rest
    
    GdkPixmap *pixmap_pc;
    GdkPixmap *pixmap_break;
    GtkItemFactory *popup_item_factory; // popup menu in .asm and disassembly
};

typedef struct _SourceBrowserAsm_Window SourceBrowserAsm_Window;


//
// The Source Opcode Browser Data
//
struct _SourceBrowserOpcode_Window {
    SourceBrowser_Window sbw;

    GtkWidget *clist;
    int clist_rows;      // Number of rows in the clist
    int current_row;     // row number of clist that corresponds
                         // to the PC of the pic.

    char **column_titles; //
    int  columns;         //
};

typedef struct _SourceBrowserOpcode_Window SourceBrowserOpcode_Window;


//
// Future Items that will be declared.
//
/*
struct _Stack_Window {
  GUI_Processor *gp;
  stack data...
};

struct _sfr_window {
  GUI_Processor *gp;
  sfr data ...
};

struct watch_window {
  GUI_Processor *gp;
  watch data ...
};


*/


//
//  The gui_processor structure ties the gui window(s)
// to a pic that is being simulated.
//

struct _gui_processor {
  Register_Window *register_window;
  StatusBar_Window *status_bar;
  SourceBrowser_Window *program_memory;
  SourceBrowser_Window *source_browser;
  // GtkWidget *stack_window;
  // GtkWidget *sfr_window;
  // GtkWidget *watch_window;

  // Doubly-linked lists keep track of all of the windows
  // associated with this processor. 
  GList *misc_windows;
  GList *source_windows;
  GList *data_windows;

  // The pic that's associated with the gui
  pic_processor *p;

};

//
// External references and function prototypes
//
extern GtkStyle *normal_style;
extern GtkStyle *current_line_number_style;
extern GtkStyle *breakpoint_line_number_style;
extern void gp_add_window_to_list(GUI_Processor *gp, GUI_Object *go);

#endif // __GUI_H__
